%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  MATLAB Code for                                                  %
%                                                                   %
%  Multi-Objective Particle Swarm Optimization (MOPSO)              %
%  Version 1.0 - Feb. 2011                                          %
%                                                                   %
%  According to:                                                    %
%  Carlos A. Coello Coello et al.,                                  %
%  "Handling Multiple Objectives with Particle Swarm Optimization," %
%  IEEE Transactions on Evolutionary Computation, Vol. 8, No. 3,    %
%  pp. 256-279, June 2004.                                          %
%                                                                   %
%  Developed Using MATLAB R2009b (Version 7.9)                      %
%                                                                   %
%  Programmed By: S. Mostapha Kalami Heris                          %
%                                                                   %
%         e-Mail: sm.kalami@gmail.com                               %
%                 kalami@ee.kntu.ac.ir                              %
%                                                                   %
%       Homepage: http://www.kalami.ir                              %
%                                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
clear;
close all;
final_result =[];
for n = 1:50
    tic
%% Problem Definition
input_file = load('KUR.txt');
true_optimal_front = load('kur_front.txt');
objectives = 2;
[nVar c]= size(input_file);
clear c;
VarSize = [1 nVar];
VarMin = input_file(1,1);
VarMax = input_file(1,2);

VelMax=(VarMax-VarMin)/10;

%% MOPSO Settings

nPop = 10;   % Population Size

nRep = 100;   % Repository Size

MaxIt = 100;  % Maximum Number of Iterations

phi1 = 2.05;
phi2 = 2.05;
phi = phi1+phi2;
chi=2/(phi-2+sqrt(phi^2-4*phi));

w = chi;              % Inertia Weight
wdamp = 1;            % Inertia Weight Damping Ratio
c1 = chi*phi1;        % Personal Learning Coefficient
c2 = chi*phi2;        % Global Learning Coefficient

alpha = 0.1;  % Grid Inflation Parameter

nGrid = 10;   % Number of Grids per each Dimension

beta = 4;     % Leader Selection Pressure Parameter

gamma = 2;    % Extra (to be deleted) Repository Member Selection Pressure

%% Initialization

particle = CreateEmptyParticle(nPop);

for i = 1:nPop
    particle(i).Velocity = 0;
    
    particle(i).Position = unifrnd(VarMin,VarMax,VarSize);

    particle(i).Cost = KUR(particle(i).Position,nVar);

    particle(i).Best.Position = particle(i).Position;

    particle(i).Best.Cost = particle(i).Cost;
end

particle = DetermineDomination(particle);

rep = GetNonDominatedParticles(particle);

rep_costs = GetCosts(rep);

G = CreateHypercubes(rep_costs,nGrid,alpha);

for i=1:numel(rep)
    [rep(i).GridIndex rep(i).GridSubIndex] = GetGridIndex(rep(i),G);
end
    
%% MOPSO Main Loop

for it = 1:MaxIt
%     tic
    for i = 1:nPop
        rep_h = SelectLeader(rep,beta);

        particle(i).Velocity = w*particle(i).Velocity ...
                             +c1*rand*(particle(i).Best.Position - particle(i).Position) ...
                             +c2*rand*(rep_h.Position -  particle(i).Position);

        particle(i).Velocity = min(max(particle(i).Velocity,-VelMax),+VelMax);

        particle(i).Position = particle(i).Position + particle(i).Velocity;

        flag =(particle(i).Position < VarMin | particle(i).Position>VarMax);
        
        particle(i).Velocity(flag)= -particle(i).Velocity(flag);
        
        particle(i).Position = min(max(particle(i).Position,VarMin),VarMax);

        particle(i).Cost = KUR(particle(i).Position,nVar);

        if Dominates(particle(i),particle(i).Best)
            particle(i).Best.Position = particle(i).Position;
            particle(i).Best.Cost = particle(i).Cost;
            
        elseif ~Dominates(particle(i).Best,particle(i))
            if rand<0.5
                particle(i).Best.Position = particle(i).Position;
                particle(i).Best.Cost = particle(i).Cost;
            end
        end
    end
    particle = DetermineDomination(particle);
    nd_particle = GetNonDominatedParticles(particle);
    rep=[rep
         nd_particle];
    
    rep = DetermineDomination(rep);
    rep = GetNonDominatedParticles(rep);
    
    for i=1:numel(rep)
        [rep(i).GridIndex rep(i).GridSubIndex]= GetGridIndex(rep(i),G);
    end
    if numel(rep)> nRep
        EXTRA = numel(rep)-nRep;
        rep = DeleteFromRep(rep,EXTRA,gamma);
        
        rep_costs=GetCosts(rep);
        G = CreateHypercubes(rep_costs,nGrid,alpha);
        
    end
   
    if ~mod(it,10)
                clc
                fprintf('%d Iteration in progress\n',n);
                fprintf('%d generations completed\n',it);
    end
    w = w*wdamp;
%     iteration_time(it)= toc;
end
    generational_distance_metic(n,1) = generational_distance(rep,true_optimal_front,objectives);
    diversity(n,1) = deversity_metric(rep,true_optimal_front,objectives);
    rr1 = numel(rep);
    rrr1 = numel(final_result);
    advances = rr1+rrr1;
    for q =1:rr1
        final_result(rrr1+q,:).Position = rep(q).Best;
        final_result(rrr1+q,:).Velocity = rep(q).Velocity;
        final_result(rrr1+q,:).Cost = rep(q).Cost;
        final_result(rrr1+q,:).Dominated = rep(q).Dominated;
        final_result(rrr1+q,:).Best.Position = rep(q).Best.Position;
        final_result(rrr1+q,:).Best.Cost = rep(q).Best.Cost;
        final_result(rrr1+q,:).GridIndex = rep(q).GridIndex;
        final_result(rrr1+q,:).GridSubIndex = rep(q).GridSubIndex;
    end
    iteration_time(n)= toc;
end
    final_result = DetermineDomination(final_result);
    nd_final_result = GetNonDominatedParticles(final_result);    
    rep=[rep
            nd_final_result];    
    rep = DetermineDomination(rep);
    rep = GetNonDominatedParticles(rep);

    gen_distance(1,1) = min(generational_distance_metic);
    gen_distance(2,1) = max(generational_distance_metic);
    gen_distance(3,1) = mean(generational_distance_metic);
    gen_distance(4,1) = median(generational_distance_metic);
    gen_distance(5,1) = std(generational_distance_metic);
    
    diversity_distance(1,1) = min(diversity);
    diversity_distance(2,1) = max(diversity);
    diversity_distance(3,1) = mean(diversity);
    diversity_distance(4,1) = median(diversity);
    diversity_distance(5,1) = std(diversity);
        
    iter_time(1,1) = min(iteration_time);
    iter_time(2,1) = max(iteration_time);
    iter_time(3,1) = mean(iteration_time);
    iter_time(4,1) = median(iteration_time);
    iter_time(5,1) = std(iteration_time);
    rep_costs = GetCosts(rep);
%% Result
% Save the result in ASCII text format.
save gen_distance.txt gen_distance -ASCII
save diversity_distance.txt diversity_distance -ASCII
save iteration_time.txt iter_time -ASCII
save MOPSO_KUR_2000.txt rep_costs -ASCII
%% Plot
figure;
f1 = rep_costs(1,:);
f2 = rep_costs(2,:);
plot(f1,f2,'rx','LineWidth',1.5,...
                'MarkerEdgeColor','r',...
                'MarkerFaceColor',[1 1 1],...
                'MarkerSize',5)
hold on
true_optimal_front = sortrows(true_optimal_front,1);
f1 = true_optimal_front(:,1);
f2 = true_optimal_front(:,2);
plot(f1,f2,'-k','LineWidth',1.5,...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor',[1 1 1],...
                'MarkerSize',5)
h = legend('MOPSO','kur_optimal',0);
set(h,'Interpreter','none')   
xlabel('Objective Function 1')
ylabel('Objective Function 2')